// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// %                                                                   %
// %   IsoCSG Library - version 0.6                                    %
// %                                                                   %
// %   iso_rcsg.inc - Variation with rounded shape functions           %
// %                                                                   %
// %   developed March 2002 by ABX <abx@abx.art.pl>                    %
// %                                                                   %
// %   based on isoCSG written June 2001 - March 2002                  %
// %   by Christoph Hormann <chris_hormann@gmx.de>                     %
// %                                                                   %
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// %                                                                   %
// %   Feel free to use this file for your povray work, I encourage    %
// %   you to contact me if you have ideas for improvements.           %
// %                                                                   %
// %   This is an experimental version so syntax and used methods      %
// %   are subject to changes.                                         %
// %                                                                   %
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#ifdef(iso_RCSG_Inc_Temp)
// do nothing
#else
#declare iso_RCSG_Inc_Temp = version;

#ifdef(View_POV_Include_Stack)
#   debug "including iso_rcsg.inc\n"
#end

#include "math.inc"
#include "transforms.inc"
#include "functions.inc"

#ifndef(iso_CSG_Inc_Temp)
  #include "iso_csg.inc"
#end

//-------------------------------------------------------------
// ----   Shape macros   --------------------------------------
//-------------------------------------------------------------

#macro IC_Box( _ICSG_Start , _ICSG_End )
  #local Start=_ICSG_Start+<0,0,0>;
  #local End=_ICSG_End+<0,0,0>;

  #local Cx=(Start.x + End.x)/2;
  #local Cy=(Start.y + End.y)/2;
  #local Cz=(Start.z + End.z)/2;
  #local Dx=abs(Start.x - End.x)/2;
  #local Dy=abs(Start.y - End.y)/2;
  #local Dz=abs(Start.z - End.z)/2;

  #local Inside=function{pattern{object{box{ Start , End }}}};

  function {
    select(
      Inside(x,y,z),
      0,
      f_rounded_box(x-Cx, y-Cy, z-Cz, 0, Dx, Dy, Dz),
      max(x-(Cx+Dx), y-(Cy+Dy), z-(Cz+Dz), (Cx-Dx)-x, (Cy-Dy)-y, (Cz-Dz)-z)
    )
  }
#end  

#macro IC_Round_Box( _ICSG_Start , _ICSG_End , _ICSG_Radius )
  #local Start=_ICSG_Start+<0,0,0>;
  #local End=_ICSG_End+<0,0,0>;

  #local Cx=(Start.x + End.x)/2;
  #local Cy=(Start.y + End.y)/2;
  #local Cz=(Start.z + End.z)/2;
  #local Dx=abs(Start.x - End.x)/2;
  #local Dy=abs(Start.y - End.y)/2;
  #local Dz=abs(Start.z - End.z)/2;

  function {
    f_rounded_box(x-Cx, y-Cy, z-Cz, _ICSG_Radius, Dx, Dy, Dz)
  }
#end

#macro IC_Cylinder (_ICSG_Start, _ICSG_End, _ICSG_Radius)
  #local Center = <0,0,0>+(_ICSG_Start+_ICSG_End)/2;

  #local Cx=Center.x;
  #local Cy=Center.y;
  #local Cz=Center.z;

  #local Len=vlength(_ICSG_End-Center);

  #if (Len=0)
    #error "iso_csg.inc: Error: zero height cylinder"
  #end

  #local fn_T=function { Reorient_Trans(_ICSG_End-Center, x)  }

  #local fn_X=
  function(x, y){
    select(x,max(x,y),select(y,max(x,y),f_r(x,y,0)))
  }
  #local fn_Y=
  function(x, y, z){
    fn_X( abs(fn_T(x, y, z).x)-Len, f_r(0,fn_T(x, y, z).y,fn_T(x, y, z).z)-_ICSG_Radius)
  }

  function {
    (fn_Y((x-Cx),(y-Cy),(z-Cz)))
  }
#end

#macro IC_Cone (_ICSG_Start, _ICSG_Radius1, _ICSG_End, _ICSG_Radius2)
  #switch(select(_ICSG_Radius1-_ICSG_Radius2,-1,0,1))
    #case(0)
      IC_Cylinder (_ICSG_Start, _ICSG_End, _ICSG_Radius1)
    #break
    #case(-1)
      #local temp=_ICSG_Start;
      #local _ICSG_Start=_ICSG_End;
      #local _ICSG_End=temp;
      #local temp=_ICSG_Radius1;
      #local _ICSG_Radius1=_ICSG_Radius2;
      #local _ICSG_Radius2=temp;
    #else
      #local _ICSG_Start=_ICSG_Start+<0,0,0>;

      #local Cx=_ICSG_Start.x;
      #local Cy=_ICSG_Start.y;
      #local Cz=_ICSG_Start.z;

      #local Len=vlength(_ICSG_End-_ICSG_Start);

      #if (Len=0)
        #error "iso_csg.inc: Error: zero height cone"
      #end

      #local fn_T=function { Reorient_Trans(_ICSG_End-_ICSG_Start, x)  }

      #local Edge=vnormalize(<Len,_ICSG_Radius1-_ICSG_Radius2,0>);
      #local Ex=Edge.x;
      #local Ey=Edge.y;

      #local fn_X=
        function(x, y){
          select(
            x,
            select(
              y-_ICSG_Radius1,
              -x,                                         // under bottom
              f_r(x,y-_ICSG_Radius1,0)                    // bottom rounded edge below bootom plane
            ),
            select(
              x*Ex-Ey*(y-_ICSG_Radius1),
              f_r(x,y-_ICSG_Radius1,0),                   // bottom rounded edge above bootom plane
              select(
                x*Ey+y*Ex-_ICSG_Radius1*Ex,
                max(-x,x-Len,x*Ey+y*Ex-_ICSG_Radius1*Ex), // inside and above top cone
                select(
                  x*Ex-Ey*y-(Ex*Len-Ey*_ICSG_Radius2),
                  x*Ey+y*Ex-_ICSG_Radius1*Ex,             // outside edge
                  select(
                    y-_ICSG_Radius2,
                    x-Len,                                // above top
                    f_r(x-Len,y-_ICSG_Radius2,0)          // top rounded edge
                  )
                )
              )
            )
          )
        }

      #local fn_Y=
        function(x, y, z){
          fn_X( fn_T(x, y, z).x, f_r(0,fn_T(x, y, z).y,fn_T(x, y, z).z))
        }

      function {
        (fn_Y((x-Cx),(y-Cy),(z-Cz)))
      }
  #end
#end

//-------------------------------------------------------------


#version iso_RCSG_Inc_Temp;
#end
